---
title: "Programmation séquentielle"
subtitle: "Série 2 -  Le nombre secret"
date: 20.09.2022
---

# Buts

- Introduction au langage C
- Utilisation des boucles et des tests
- Utilisation des entrées-sorties
- Utilisation de nombres aléatoires

# Énoncé

Écrire un programme permettant de jouer au nombre secret. Le but est de faire
chercher à l’utilisateur·trice un nombre déterminé aléatoirement par
l'ordinateur. Le nombre de coups mis pour trouver la solution sera compté et
affiché en fin de la partie. La borne maximum pour le choix du nombre caché
sera demandée à l’utilisateur·trice.

Pour générer des nombres aléatoires, utiliser la fonction  `rand()`{.C}  qui retourne un nombre aléatoire entre 0 et l'entier maximum (cf. la commande `man 3 rand`). Pour initialiser le générateur de nombre aléatoire, appeler la fonction `srand()`{.C} qui prend en paramètre un entier non-signé(la graine). On peut par exemple passer `time(NULL)`{.C} en paramètre à `srand()`{.C}. L'appel `time(NULL)`{.C} retourne l'horloge système en secondes.

\newpage

Voici un exemple de programme en C permettant de générer des nombres aléatoires :

```C
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

void main(){
  srand(time(NULL));
  for(int i=0; i<5; i++){
    int n = rand();
    printf("Nombre généré : %d\n", n);
  }
}
```

Il est nécessaire d'inclure les bibliothèques suivantes dans le programme:

```C
#include <stdio.h>      /* printf(), scanf() */
#include <stdlib.h>     /* srand(), rand() */
#include <time.h>       /* time() */
#include <math.h>       /* log() */
```

A la compilation il faut également ajouter le flag `-lm` pour générer correctement l'exécutable.

# Déroulement du programme

- Au début du programme, l’utilisateur·trice entre le nombre maximum avec lequel il veut jouer. 
- Le programme générera un nombre pris au hasard entre 0 et ce maximum.
- Dans une boucle, le programme demandera à l’utilisateur·trice d’entrer un nombre, puis lui répondra par plus petit ou plus grand et ceci jusqu’à la découverte du nombre secret.
- Afin de s'assurer que l'utilisateur·trice joue selon les règles, le programme
  doit vérifier que le nombre entré par l'utilisateur·trice n'est pas plus
  petit que zéro ou plus grand que le nombre maximal autorisé. Si un de ces
  deux cas se présente, le programme affichera un message d'avertissement à
  l'utilisateur·trice[^1] afin de lui rappeler les règles.
- Afficher en fin de partie le nombre de coups mis par le joueur pour découvrir le nombre secret

# Partie bonus

Cette partie n'est pas obligatoire. Vous pouvez la faire si vous souhaitez aller plus loin.

## Nombre de coup optimal

Chercher le moyen de trouver le nombre caché de manière optimale en moyenne. Afficher le nombre de coups optimal maximal théorique permettant de déterminer le nombre secret.

## L'ordinateur joue contre l'utilisateur

Une fois ce qui précède terminé, échangez les rôles!

Écrire un programme où l'ordinateur et l'utilisateur·trice échangent leurs
rôles. Afin de signifier à l'ordinateur que le nombre est plus grand ou plus
petit, l'utilisateur·trice rentre `<` ou `>` respectivement et `=` en cas de victoire de
l'ordinateur (il se pourrait que le type `char` vous soit utile). 
Implémenter différentes stratégies pour l'ordinateur (recherche
exhaustive, bisection, etc.).

[^1]: Par exemple: "Vous devez entrer un nombre plus grand ou égal à zéro." ou "Vous devez entrer un nombre plus petit que 'max'.".
